@@ -54,8 +54,9 @@ class Perf:
         self.profiler_process_stderr = None
         self.profiler_process_exit_code = None
         self.trace_file = None
-        self.collapsed_stack_file = None
+        self.trace_file_main_thread = None
         self.stack_collapse_file = None
+        self.stack_collapse_file_main_thread = None
         self.collapsed_stacks = []
         self.pid = None
         self.started_profile = False
@@ -67,9 +68,13 @@ class Perf:
         self.retrieve_perf_version()
         self.profile_start_time = None
         self.profile_end_time = None
+        self.logger = logging
 
         self.pprof_bin = whereis("pprof")
 
+    def set_logger(self, logger_app):
+        self.logger = logger_app
+
     def retrieve_perf_version(self):
         try:
             self.version = subprocess.Popen(
@@ -103,7 +108,9 @@ class Perf:
             cmd += ["--freq", "{}".format(frequency)]
         return cmd
 
-    def generate_report_command(self, tid, input, dso, percentage_mode):
+    def generate_report_command(
+        self, tid, input, dso, percentage_mode, extra_options=None
+    ):
         cmd = [self.perf, "report"]
         if dso is not None:
             cmd += ["--dso", dso]
@@ -113,13 +120,20 @@ class Perf:
             "{}".format(tid),
             "--no-children",
             "--stdio",
-            "-g",
-            "none,1.0,caller,function",
             "--percentage",
             percentage_mode,
             "--input",
             input,
         ]
+        if extra_options is None:
+            cmd.extend(
+                [
+                    "-g",
+                    "none,1.0,caller,function",
+                ]
+            )
+        elif type(extra_options) == list:
+            cmd.extend(extra_options)
         return cmd
 
     def start_profile(self, pid, output, frequency=99):
@@ -146,7 +160,9 @@ class Perf:
             }
 
             args = self.generate_record_command(pid, output, frequency)
-            logging.info("Starting profile of pid {} with args {}".format(pid, args))
+            self.logger.info(
+                "Starting profile of pid {} with args {}".format(pid, args)
+            )
             self.profiler_process = subprocess.Popen(args=args, **options)
             self.started_profile = True
             result = True
@@ -161,8 +177,10 @@ class Perf:
             return False
         # Check if child process has terminated. Set and return returncode
         # attribute
-        if process.poll() is None:
+        ret = process.poll()
+        if ret is None:
             return True
+        self.profiler_process_exit_code = ret
         return False
 
     def stop_profile(self, **kwargs):
@@ -172,8 +190,20 @@ class Perf:
         result = False
         self.profile_end_time = time.time()
         if not self._is_alive(self.profiler_process):
-            logging.error(
-                "Profiler process is not alive, might have crash during test execution, "
+            self.logger.error(
+                "Profiler process is not alive, might have crash during test execution.  Exit code: {}".format(
+                    self.profiler_process_exit_code
+                )
+            )
+            (
+                self.profiler_process_stdout,
+                self.profiler_process_stderr,
+            ) = self.profiler_process.communicate()
+            self.logger.error(
+                "Profiler stderr: {}".format(self.profiler_process_stderr)
+            )
+            self.logger.error(
+                "Profiler stdout: {}".format(self.profiler_process_stdout)
             )
             return result
         try:
@@ -185,21 +215,23 @@ class Perf:
             ) = self.profiler_process.communicate()
             self.profiler_process_exit_code = self.profiler_process.poll()
             if self.profiler_process_exit_code <= 0:
-                logging.info("Generating trace file from profile.")
+                self.logger.info("Generating trace file from profile.")
                 result = self.generate_trace_file_from_profile()
                 if result is True:
-                    logging.info("Trace file generation ran OK.")
+                    self.logger.info("Trace file generation ran OK.")
                     result = self.stack_collapse()
                     if result is True:
-                        logging.info("Stack collapsing from trace file ran OK.")
+                        self.logger.info("Stack collapsing from trace file ran OK.")
                     else:
-                        logging.error(
+                        self.logger.error(
                             "Something went wrong on stack collapsing from trace file."
                         )
                 else:
-                    logging.error("Stack collapsing from trace file exit with error.")
+                    self.logger.error(
+                        "Stack collapsing from trace file exit with error."
+                    )
             else:
-                logging.error(
+                self.logger.error(
                     "Profiler process exit with error. Exit code: {}\n\n".format(
                         self.profiler_process_exit_code
                     )
@@ -207,7 +239,7 @@ class Perf:
                 )
 
         except OSError as e:
-            logging.error(
+            self.logger.error(
                 "OSError caught while waiting for profiler process to end: {0}".format(
                     e.__str__()
                 )
@@ -248,9 +280,33 @@ class Perf:
                     try:
                         subprocess.Popen(args=args, stdout=outfile).wait()
                     except OSError as e:
-                        logging.error(
+                        self.logger.error(
                             "Unable to run {} script {}".format(self.perf, e.__str__())
                         )
+                if self.pid is not None:
+                    filename_main_thread = filename + ".mainthread"
+                    with open(filename_main_thread, "w") as outfile:
+                        args = [
+                            self.perf,
+                            "script",
+                            "--tid",
+                            self.pid,
+                            "-i",
+                            self.output,
+                        ]
+                        try:
+                            subprocess.Popen(args=args, stdout=outfile).wait()
+                            self.trace_file_main_thread = filename_main_thread
+                        except OSError as e:
+                            self.logger.error(
+                                "Unable to run {} script {}".format(
+                                    self.perf, e.__str__()
+                                )
+                            )
+                else:
+                    logging.warning(
+                        "Unable to generate main thread only collapsed stack files given no PID data is available"
+                    )
                 result = True
                 self.trace_file = filename
 
@@ -267,7 +323,7 @@ class Perf:
                     try:
                         subprocess.Popen(args=args, stdout=outfile).wait()
                     except OSError as e:
-                        logging.error(
+                        self.logger.error(
                             "Unable to stack collapse using: {0} {1}. Error {2}".format(
                                 self.stack_collapser, self.trace_file, e.__str__()
                             )
@@ -275,16 +331,41 @@ class Perf:
                 self.stack_collapse_file = filename
                 result = True
             else:
-                logging.error("Unable to open {0}".format(self.trace_file))
+                self.logger.error("Unable to open {0}".format(self.trace_file))
+        if self.trace_file_main_thread is not None:
+            if os.path.isfile(self.trace_file_main_thread):
+                filename = self.output + ".mainthread.stacks-folded"
+                with open(filename, "w") as outfile:
+                    args = [
+                        self.stack_collapser,
+                        os.path.abspath(self.trace_file_main_thread),
+                    ]
+                    try:
+                        subprocess.Popen(args=args, stdout=outfile).wait()
+                    except OSError as e:
+                        self.logger.error(
+                            "Unable to stack collapse using: {0} {1}. Error {2}".format(
+                                self.stack_collapser, self.trace_file, e.__str__()
+                            )
+                        )
+                self.stack_collapse_file_main_thread = filename
+                result = True
+            else:
+                self.logger.error("Unable to open {0}".format(self.trace_file))
         return result
 
     def generate_outputs(self, use_case, **kwargs):
         outputs = {}
+        tabular_data_map = {}
         binary = kwargs.get("binary")
         details = kwargs.get("details")
         primary_id = kwargs.get("primary_id")
         total_primaries = kwargs.get("total_primaries")
-        identifier = "primary {} of {}".format(primary_id, total_primaries)
+        identifier = "primary_{}_of_{}".format(primary_id, total_primaries)
+
+        # If we have only one primary there is no need to bloat the description of artifacts
+        if primary_id == 1 and primary_id == total_primaries:
+            identifier = ""
         if details is None:
             details = ""
         result = True
@@ -293,18 +374,134 @@ class Perf:
             "Flame Graph: " + use_case, details
         )
         if artifact_result is True:
-            outputs["Flame Graph ({})".format(identifier)] = flame_graph_output
+            outputs["Flame Graph {}".format(identifier)] = flame_graph_output
         result &= artifact_result
 
-        # save perf output
+        # generate main thread flame graph
+        artifact_result, flame_graph_output = self.generate_flame_graph_main_thread(
+            "Main THREAD Flame Graph: " + use_case, details
+        )
         if artifact_result is True:
-            outputs["perf output ({})".format(identifier)] = os.path.abspath(
-                self.output
-            )
+            outputs[
+                "Main THREAD Flame Graph {}".format(identifier)
+            ] = flame_graph_output
+        result &= artifact_result
 
         tid = self.pid
+
+        # generate perf report per dso
+        self.logger.info(
+            "Generating perf report per name of library or module executed at the time of sample"
+        )
+        perf_report_output = self.output + ".perf-report.dso.txt"
+
+        artifact_result, perf_report_artifact = self.run_perf_report(
+            tid,
+            perf_report_output,
+            None,
+            "absolute",
+            ["--percent-limit", "1", "-s", "dso", "-q", "--call-graph=none"],
+        )
+
+        if artifact_result is True:
+            outputs["perf report per dso {}".format(identifier)] = perf_report_artifact
+        result &= artifact_result
+
+        # generate perf report per dso,sym
+        self.logger.info(
+            "Generating perf report per name of function executed at the time of sample"
+        )
+        perf_report_output = self.output + ".perf-report.dso+sym.txt"
+
+        artifact_result, perf_report_artifact = self.run_perf_report(
+            tid,
+            perf_report_output,
+            None,
+            "absolute",
+            ["--percent-limit", "1", "-s", "dso,sym", "-q", "--call-graph=none"],
+        )
+
+        if artifact_result is True:
+            outputs[
+                "perf report per dso,sym {}".format(identifier)
+            ] = perf_report_artifact
+        result &= artifact_result
+
+        # generate perf report per dso,sym
+        self.logger.info(
+            "Generating perf report per name of function executed at the time of sample with callgraph"
+        )
+        perf_report_output = self.output + ".perf-report.dso+sym.callgraph.txt"
+
+        artifact_result, perf_report_artifact = self.run_perf_report(
+            tid,
+            perf_report_output,
+            None,
+            "absolute",
+            ["--percent-limit", "1", "-s", "dso,sym", "-q", "--call-graph=flat"],
+        )
+
+        if artifact_result is True:
+            outputs[
+                "perf report per dso,sym with callgraph {}".format(identifier)
+            ] = perf_report_artifact
+        result &= artifact_result
+
+        # generate perf report per dso,sym,srcline
+        self.logger.info(
+            "Generating perf report per filename and line number executed at the time of sample"
+        )
+        perf_report_output = self.output + ".perf-report.dso+sym+srcline.txt"
+
+        artifact_result, perf_report_artifact = self.run_perf_report(
+            tid,
+            perf_report_output,
+            None,
+            "absolute",
+            [
+                "--percent-limit",
+                "1",
+                "-s",
+                "dso,sym,srcline",
+                "-q",
+                "--call-graph=none",
+            ],
+        )
+
+        if artifact_result is True:
+            outputs[
+                "perf report per dso,sym,srcline {}".format(identifier)
+            ] = perf_report_artifact
+        result &= artifact_result
+
+        self.logger.info(
+            "Generating perf report per filename and line number executed at the time of sample with callgraph"
+        )
+        perf_report_output = self.output + ".perf-report.dso+sym+srcline.callgraph.txt"
+
+        artifact_result, perf_report_artifact = self.run_perf_report(
+            tid,
+            perf_report_output,
+            None,
+            "absolute",
+            [
+                "--percent-limit",
+                "1",
+                "-s",
+                "dso,sym,srcline",
+                "-q",
+                "--call-graph=flat",
+            ],
+        )
+
+        if artifact_result is True:
+            outputs[
+                "perf report per dso,sym,srcline with callgraph {}".format(identifier)
+            ] = perf_report_artifact
+        result &= artifact_result
+
         # generate perf report --stdio report
-        logging.info("Generating perf report text outputs")
+        self.logger.info("Generating perf report text outputs")
         perf_report_output = self.output + ".perf-report.top-cpu.txt"
 
         artifact_result, perf_report_artifact = self.run_perf_report(
@@ -313,13 +510,13 @@ class Perf:
 
         if artifact_result is True:
             outputs[
-                "perf report top self-cpu ({})".format(identifier)
+                "perf report top self-cpu {}".format(identifier)
             ] = perf_report_artifact
         result &= artifact_result
 
         # generate perf report --stdio report
         if binary is not None:
-            logging.info(
+            self.logger.info(
                 "Generating perf report text outputs only for dso only ({})".format(
                     identifier
                 )
@@ -337,34 +534,51 @@ class Perf:
         result &= artifact_result
 
         if self.callgraph_mode == "dwarf":
-            logging.warning(
+            self.logger.warning(
                 "Unable to use perf output collected with callgraph dwarf mode in pprof. Skipping artifacts generation."
             )
-            logging.warning(
+            self.logger.warning(
                 "Check https://github.com/google/perf_data_converter/issues/40."
             )
         else:
             if self.pprof_bin is None:
-                logging.error(
+                self.logger.error(
                     "Unable to detect pprof. Some of the capabilities will be disabled"
                 )
             else:
-                logging.info("Generating pprof text output")
+                self.logger.info("Generating pprof text output")
                 pprof_text_output = self.output + ".pprof.txt"
-                artifact_result, pprof_artifact_text_output = run_pprof(
+                artifact_result, pprof_artifact_text_output, tabular_data = run_pprof(
                     self.pprof_bin,
                     PPROF_FORMAT_TEXT,
                     pprof_text_output,
                     binary,
                     self.output,
                 )
-                result &= artifact_result
 
+                result &= artifact_result
                 if artifact_result is True:
                     outputs["Top entries in text form"] = pprof_artifact_text_output
-                logging.info("Generating pprof png output")
+                    tabular_data_map["text"] = tabular_data
+
+                self.logger.info("Generating pprof per LOC text output")
+                pprof_text_output = self.output + ".pprof.LOC.txt"
+                artifact_result, pprof_artifact_text_output, tabular_data = run_pprof(
+                    self.pprof_bin,
+                    [PPROF_FORMAT_TEXT, "-lines"],
+                    pprof_text_output,
+                    binary,
+                    self.output,
+                )
+                result &= artifact_result
+                if artifact_result is True:
+                    outputs[
+                        "Top entries in text form by LOC"
+                    ] = pprof_artifact_text_output
+                    tabular_data_map["text-lines"] = tabular_data
+                self.logger.info("Generating pprof png output")
                 pprof_png_output = self.output + ".pprof.png"
-                artifact_result, pprof_artifact_png_output = run_pprof(
+                artifact_result, pprof_artifact_png_output, _ = run_pprof(
                     self.pprof_bin,
                     PPROF_FORMAT_PNG,
                     pprof_png_output,
@@ -377,7 +591,15 @@ class Perf:
                     ] = pprof_artifact_png_output
                 result &= artifact_result
 
-        return result, outputs
+        # save stack collapsed
+        outputs["Identical stacks collapsed {}".format(identifier)] = os.path.abspath(
+            self.stack_collapse_file
+        )
+
+        # save perf output
+        outputs["perf output {}".format(identifier)] = os.path.abspath(self.output)
+
+        return result, outputs, tabular_data_map
 
     def generate_flame_graph(self, title="Flame Graph", subtitle="", filename=None):
         result = False
@@ -402,7 +624,7 @@ class Perf:
                     try:
                         subprocess.Popen(args=args, stdout=outfile).wait()
                     except OSError as e:
-                        logging.error(
+                        self.logger.error(
                             "Unable t use flamegraph.pl to render a SVG. using: {0} {1}. Error {2}".format(
                                 self.flamegraph_utity,
                                 self.stack_collapse_file,
@@ -411,32 +633,78 @@ class Perf:
                         )
                 result = True
                 result_artifact = os.path.abspath(filename)
-                logging.info(
+                self.logger.info(
                     "Successfully rendered flame graph to {0}".format(result_artifact)
                 )
             else:
-                logging.error("Unable to open {0}".format(self.stack_collapse_file))
+                self.logger.error("Unable to open {0}".format(self.stack_collapse_file))
+        return result, result_artifact
+
+    def generate_flame_graph_main_thread(
+        self, title="MAIN THREAD Flame Graph", subtitle="", filename=None
+    ):
+        result = False
+        result_artifact = None
+        # total_profile_duration = self.profile_end_time - self.profile_start_time
+
+        if self.stack_collapse_file_main_thread is not None:
+            if os.path.isfile(self.stack_collapse_file_main_thread):
+                if filename is None:
+                    filename = self.output + ".mainthread.flamegraph.svg"
+                with open(filename, "w") as outfile:
+                    args = [
+                        self.flamegraph_utity,
+                        "--title",
+                        title,
+                        "--subtitle",
+                        subtitle,
+                        "--width",
+                        "1600",
+                        os.path.abspath(self.stack_collapse_file_main_thread),
+                    ]
+                    try:
+                        subprocess.Popen(args=args, stdout=outfile).wait()
+                    except OSError as e:
+                        self.logger.error(
+                            "Unable t use flamegraph.pl to render a SVG. using: {0} {1}. Error {2}".format(
+                                self.flamegraph_utity,
+                                self.stack_collapse_file_main_thread,
+                                e.__str__(),
+                            )
+                        )
+                result = True
+                result_artifact = os.path.abspath(filename)
+                self.logger.info(
+                    "Successfully rendered flame graph to {0}".format(result_artifact)
+                )
+            else:
+                self.logger.error(
+                    "Unable to open {0}".format(self.stack_collapse_file_main_thread)
+                )
         return result, result_artifact
 
     def get_collapsed_stacks(self):
         return self.collapsed_stacks
 
-    def run_perf_report(self, tid, output, dso, percentage_mode):
+    def run_perf_report(self, tid, output, dso, percentage_mode, extra_options=None):
         status = False
         result_artifact = None
-        args = self.generate_report_command(tid, self.output, dso, percentage_mode)
-        logging.info("Running {} report with args {}".format(self.perf, args))
+        args = self.generate_report_command(
+            tid, self.output, dso, percentage_mode, extra_options
+        )
+        self.logger.info("Running {} report with args {}".format(self.perf, args))
+        self.logger.info("Final command: {} {}".format(self.perf, " ".join(args)))
         try:
             stdout, _ = subprocess.Popen(
                 args=args, stdout=subprocess.PIPE
             ).communicate()
-            logging.debug(stdout)
+            self.logger.debug(stdout)
             with open(output, "w") as outfile:
                 outfile.write(stdout.decode())
             status = True
             result_artifact = os.path.abspath(output)
         except OSError as e:
-            logging.error(
+            self.logger.error(
                 "Unable to run {} report. Error {}".format(self.perf, e.__str__())
             )
         return status, result_artifact
